#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "options.h"
#include "cptinfo.h"

#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char **argv)
{
  struct gengetopt_args_info info;
  cptinfo_opt_t opt = {
    .file = {
      .input  = NULL,
      .output = NULL
    },
    .verbose = false,
    .format  = plain
  };
  char *infile, *outfile;

  /* use gengetopt */

  if (options(argc, argv, &info) != 0)
    {
      fprintf(stderr,"failed to parse command line\n");
      return EXIT_FAILURE;
    }

  /* check arguments & transfer to opt structure */

  opt.verbose = info.verbose_given;

  /* null outfile for stdout */

  if (info.output_given)
    outfile = info.output_arg;
  else
    {
      if (opt.verbose)
        {
          fprintf(stderr, "no --verbose without --output\n");
          return EXIT_FAILURE;
        }
      outfile = NULL;
    }

  /* null infile for stdin */

  switch (info.inputs_num)
    {
    case 0:
      infile = NULL;
      break;
    case 1:
      infile = info.inputs[0];
      break;
    default:
      fprintf(stderr,"Sorry, only one file at a time\n");
      return EXIT_FAILURE;
    }

  if (opt.verbose)
    printf("This is cptinfo (version %s)\n",VERSION);

  opt.file.input  = infile;
  opt.file.output = outfile;

  opt.format = plain;
  if (info.csv_given)  opt.format = csv;

  int err = cptinfo(opt);

  if (err)
    fprintf(stderr,"failed (error %i)\n",err);

  if (opt.verbose)
    printf("done.\n");

  options_free(&info);

  return (err ? EXIT_FAILURE : EXIT_SUCCESS);
}
