#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "options.h"
#include "gplcpt.h"

#include <cptutils/btrace.h>

#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char **argv)
{
  struct gengetopt_args_info info;
  gplcpt_opt_t opt = {0};
  char *infile, *outfile;

  /* use gengetopt */

  if (options(argc, argv, &info) != 0)
    {
      fprintf(stderr,"failed to parse command line\n");
      return EXIT_FAILURE;
    }

  /* check arguments & transfer to opt structure */

  opt.verbose = info.verbose_given;

  /* null outfile for stdout */

  if (info.output_given)
    outfile = info.output_arg;
  else
    {
      if (opt.verbose)
        {
          fprintf(stderr, "no --verbose without --output\n");
          return EXIT_FAILURE;
        }
      outfile = NULL;
    }

  /* null infile for stdin */

  switch (info.inputs_num)
    {
    case 0:
      infile = NULL;
      break;
    case 1:
      infile = info.inputs[0];
      break;
    default:
      fprintf(stderr, "Sorry, only one file at a time\n");
      return EXIT_FAILURE;
    }

  if (opt.verbose)
    printf("This is gplcpt (version %s)\n", VERSION);

  opt.file.input = infile;
  opt.file.output = outfile;

  /* get fg/bg/nan */

  if (parse_rgb(info.background_arg, &opt.bg) != 0)
    {
      fprintf(stderr,"bad background %s\n", info.background_arg);
      return EXIT_FAILURE;
    }

  if (parse_rgb(info.foreground_arg, &opt.fg) != 0)
    {
      fprintf(stderr,"bad foreground %s\n", info.foreground_arg);
      return EXIT_FAILURE;
    }

  if (parse_rgb(info.nan_arg, &opt.nan) != 0)
    {
      fprintf(stderr,"bad nan colour %s\n", info.nan_arg);
      return EXIT_FAILURE;
    }

  btrace_enable("gplcpt");

  int err = gplcpt(opt);

  if (err)
    {
      btrace("failed (error %i)", err);
      btrace_print_stream(stderr, BTRACE_PLAIN);

      if (info.backtrace_file_given)
	{
	  int format = BTRACE_XML;
	  if (info.backtrace_format_given)
	    {
	      format = btrace_format(info.backtrace_format_arg);
	      if (format == BTRACE_ERROR)
		{
		  fprintf(stderr, "no such backtrace format %s\n",
			  info.backtrace_format_arg);
		  return EXIT_FAILURE;
		}
	    }
	  btrace_print(info.backtrace_file_arg, format);
	}
    }

  btrace_reset();
  btrace_disable();

  if (opt.verbose)
    printf("done.\n");

  options_free(&info);

  return (err ? EXIT_FAILURE : EXIT_SUCCESS);
}
