#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cptutils/cpt-fill.h"
#include "cptutils/btrace.h"

#include <stdio.h>

int cpt_fill_eq(cpt_fill_t a, cpt_fill_t b, model_t model)
{
  if (a.type != b.type) return 0;

  switch (a.type)
    {
    case cpt_fill_empty:
      return 1;
    case cpt_fill_grey:
      return (a.grey == b.grey);
    case cpt_fill_hatch :
      return ((a.hatch.sign == b.hatch.sign) &&
	      (a.hatch.dpi == b.hatch.dpi) &&
	      (a.hatch.n == b.hatch.n));
    case cpt_fill_file:
      /* FIXME */
      return 0;
    case cpt_fill_colour_rgb:
      return colour_rgb_dist(a.colour, b.colour, model) < 1e-8;
    case cpt_fill_colour_hsv:
      return 0;
    }

  btrace("no such fill type");

  return 1;
}

int cpt_fill_interpolate(double z, cpt_fill_t a, cpt_fill_t b,
                         model_t model, cpt_fill_t *f)
{
  if (a.type != b.type)
    return 1;

  cpt_fill_type_t type = a.type;

  f->type = type;

  switch (type)
    {
    case cpt_fill_empty :
      break;

    case cpt_fill_grey :
      f->grey = (a.grey*(1-z) + b.grey*z);
      break;

    case cpt_fill_hatch :
    case cpt_fill_file :
      *f = a;
      break;

    case cpt_fill_colour_rgb:
    case cpt_fill_colour_hsv:
      if (colour_interpolate(z, a.colour, b.colour,
			     model, &(f->colour)) != 0)
	{
	  btrace("failed to interpolate colour");
	  return 1;
	}

      break;
    }

  return 0;
}

/*
  convert a fill to an rgb triplet -- often a recieving
  program will only accept a colour.
*/

int cpt_fill_rgb(cpt_fill_t fill, model_t model, rgb_t *prgb)
{
  switch (fill.type)
    {
      double rgbD[3];

    case cpt_fill_colour_rgb:
      *prgb = fill.colour.rgb;
      break;

    case cpt_fill_colour_hsv:
      hsv_to_rgbD(fill.colour.hsv, rgbD);
      rgbD_to_rgb(rgbD, prgb);
      break;

    case cpt_fill_grey:
    case cpt_fill_hatch:
    case cpt_fill_file:
    case cpt_fill_empty:

      btrace("fill type not yet implemented");
      return 1;

    default:
      btrace("bad fill type (%i)", fill.type);
      return 1;
    }

  return 0;
}
