#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cptutils/colour.h"
#include "cptutils/cpt.h"
#include "cptutils/btrace.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* number of segments */

int cpt_nseg(const cpt_t *cpt)
{
  int n = 0;

  for (cpt_seg_t *seg = cpt->segment ; seg ; seg = seg->rseg)
    n++;

  return n;
}

int cpt_zrange(const cpt_t *cpt, double z[static 2])
{
  cpt_seg_t *seg;

  if ((seg = cpt->segment) != NULL)
    {
      z[0] = seg->lsmp.val;

      while (seg->rseg)
        seg = seg->rseg;

      z[1] = seg->rsmp.val;

      return 0;
    }

  return 1;
}

bool cpt_increasing(const cpt_t *cpt)
{
  double z[2] = {0};

  cpt_zrange(cpt, z);

  return z[0] < z[1];
}

cpt_t* cpt_new(void)
{
  cpt_t *cpt;

  if ((cpt = malloc(sizeof(cpt_t))) != NULL)
    {
      cpt->name = NULL;
      cpt->segment = NULL;
      cpt->model = model_rgb;
      cpt->interpolate = model_rgb;
      cpt->fg.type = cpt_fill_empty;
      cpt->bg.type = cpt_fill_empty;
      cpt->nan.type = cpt_fill_empty;
    }

  return cpt;
}

/* add a segment to the start (leftmost) */

int cpt_prepend(cpt_seg_t *seg, cpt_t *cpt)
{
  cpt_seg_t *s = cpt->segment;

  seg->lseg = NULL;
  seg->rseg = s;

  if (s != NULL)
    s->lseg = seg;

  cpt->segment = seg;

  return 0;
}

/* add a segment to the end (rightmost) */

int cpt_append(cpt_seg_t *seg, cpt_t *cpt)
{
  cpt_seg_t *s = cpt->segment;

  if (s)
    {
      while (s->rseg != NULL)
        s = s->rseg;

      s->rseg = seg;
      seg->lseg = s;
      seg->rseg = NULL;

      return 0;
    }
  else
    return cpt_prepend(seg, cpt);
}

/* remove first segment (and return it) */

cpt_seg_t* cpt_pop(cpt_t *cpt)
{
  cpt_seg_t *s1 = cpt->segment;

  if (s1)
    {
      cpt_seg_t *s2 = s1->rseg;

      if (s2) s2->lseg = NULL;

      cpt->segment = s2;

      s1->rseg = NULL;
      s1->lseg = NULL;

      return s1;
    }

  return NULL;
}

/* remove last segment (and return it) */

cpt_seg_t* cpt_shift(cpt_t *cpt)
{
  cpt_seg_t *s1;

  s1 = cpt->segment;

  /* no segment case */

  if (!s1) return NULL;

  /* single segment case */

  if (! s1->rseg)
    {
      cpt->segment = NULL;

      s1->lseg = NULL;
      s1->rseg = NULL;

      return s1;
    }

  /* multi segment case */

  while (s1->rseg->rseg) s1 = s1->rseg;

  cpt_seg_t *s2 = s1->rseg;

  s1->rseg = NULL;

  s2->lseg = NULL;
  s2->rseg = NULL;

  return s2;
}

void cpt_destroy(cpt_t *cpt)
{
  if (cpt)
    {
      free(cpt->name);

      cpt_seg_t *seg = cpt->segment;

      while (seg)
        {
	  cpt_seg_t *next = seg->rseg;
          cpt_seg_destroy(seg);
          seg = next;
	}
    }

  free(cpt);
}

cpt_seg_t* cpt_seg_new(void)
{
  cpt_seg_t *seg;

  if ((seg = malloc(sizeof(cpt_seg_t))) != NULL)
    {
      seg->lseg = NULL;
      seg->rseg = NULL;
      seg->annote = none;
      seg->label = NULL;
    }

  return seg;
}

void cpt_seg_destroy(cpt_seg_t *seg)
{
  if (seg) free(seg->label);
  free(seg);
}
