#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cptutils/pov.h"
#include "cptutils/btrace.h"

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <stdbool.h>

static bool has_name(const pov_t *pov)
{
  return pov->name[0] != '\0';
}

int pov_write(const pov_t *pov, const char *file)
{
  FILE *st;
  time_t t;
  size_t n = pov->n;

  if (n < 2)
    {
      btrace("povray does not support %zi stops", n);
      return 1;
    }

  if (file)
    {
      st = fopen(file, "w");
      if (!st) return 1;
    }
  else st = stdout;

  t = time(NULL);

  fprintf(st, "// autogenerated povray color map \"%s\"\n",
	  (has_name(pov) ? pov->name : "[unnamed]"));
  fprintf(st, "// cptutils version %s, %s", VERSION, ctime(&t));

  if (has_name(pov))
    fprintf(st, "#declare %s =\n", pov->name);

  fprintf(st, "color_map {\n");

  for (size_t i = 0 ; i < n ; i++)
    {
      pov_stop_t stop = pov->stop[i];

      fprintf(st, "  [%7.5f color rgbf <%.4f,%.4f,%.4f,%.4f>]\n",
	      stop.z,
	      stop.rgbt[0],
	      stop.rgbt[1],
	      stop.rgbt[2],
	      stop.rgbt[3]);
    }

  fprintf(st, "}\n");

  fclose(st);

  return 0;
}
