#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "cptutils/svg-list.h"

#include <stdlib.h>

struct svg_list_t
{
  int n, alloc;
  svg_t *svg;
};

#define LIST_INI 20
#define LIST_INC 20

svg_list_t* svg_list_new(void)
{
  svg_list_t *list;

  if ((list = malloc(sizeof(svg_list_t))) != NULL)
    {
      svg_t *svg;

      if ((svg = malloc(sizeof(svg_t) * LIST_INI)) != NULL)
	{
	  list->svg = svg;
	  list->alloc = LIST_INI;
	  list->n = 0;

	  return list;
	}

      free(list);
    }

  return NULL;
}

void svg_list_destroy(svg_list_t *list)
{
  free(list->svg);
  free(list);

  return;
}

int svg_list_iterate(svg_list_t *list,
                     int (*f)(svg_t*, void*),
                     void *opt)
{
  size_t i, n = list->n;
  svg_t *svg;

  for (i = 0, svg = list->svg ; i < n ; i++, svg++)
    {
      int err = f(svg, opt);
      if (err) return err;
    }

  return 0;
}

svg_t* svg_list_select(svg_list_t *list,
                       int (*f)(svg_t*, void*),
                       void *opt)
{
  size_t i, n = list->n;
  svg_t *svg;

  for (i = 0, svg = list->svg ; i < n ; i++, svg++)
    {
      if (f(svg, opt) != 0)
        return svg;
    }

  return NULL;
}

svg_t* svg_list_svg(svg_list_t *list)
{
  size_t
    n = list->n,
    a = list->alloc;
  svg_t *svg;

  /* return next svg_t free */

  if (n < a)
    {
      list->n++;
      svg = list->svg + n;

      svg_init(svg);

      return svg;
    }

  /* not enough room, so reallocate */

  a += LIST_INC;

  svg = realloc(list->svg, sizeof(svg_t) * a);

  if (svg == NULL) return NULL;

  list->alloc = a;
  list->svg = svg;

  return svg_list_svg(list);
}

int svg_list_size(svg_list_t *list)
{
  return list->n;
}
